/***************************************************************************
 *             __________               __   ___.
 *   Open      \______   \ ____   ____ |  | _\_ |__   _______  ___
 *   Source     |       _//  _ \_/ ___\|  |/ /| __ \ /  _ \  \/  /
 *   Jukebox    |    |   (  <_> )  \___|    < | \_\ (  <_> > <  <
 *   Firmware   |____|_  /\____/ \___  >__|_ \|___  /\____/__/\_ \
 *                     \/            \/     \/    \/            \/
 ****************************************************************************/

#include "lcd.h"
#include "button.h"
#include "kernel.h"
#include "version.h"
#include "debug_menu.h"
#include "sprintf.h"
#include <string.h>

#ifdef HAVE_LCD_BITMAP
#include "bmp.h"
#include "icons.h"
#endif

#define KEYBOARD_LINES 3
#define KEYBOARD_BUFFER_SIZE 255

void kbd_setupkeys(char* line[KEYBOARD_LINES], bool caps)
{
	if (caps)
	{
		line[0] = "QWERTYUIOP 789-_";
		line[1] = "ASDFGHJKL +456:=";
		line[2] = " ZXCVBNM < 1230'";
	}
	else
	{
		line[0] = "qwertyuiop &*(/\\";
		line[1] = "asdfghjkl? $%),;";
		line[2] = " zxcvbnm < !@#.\"";
	}
}

void kbd_draw_statusbar_button(int num, char* caption, int y, int fw)
{
	int x, x2, tw, cx;
	x = num*(LCD_WIDTH/3);
	x2 = (num+1)*(LCD_WIDTH/3);
	tw = fw * strlen(caption);
	cx = x2 - x;
	/* center the text */
	lcd_putsxy((x + (cx/2)) - (tw/2), y, caption, 0);
	lcd_invertrect(x, y - 1, (x2-x)-1, LCD_HEIGHT-y+1);
}

void center_text(char* caption, bool largefont)
{
	int font_w = 0, font_h = 0, font_w1 = 0, font_h1 = 0, n, curroffset = 0, MAX_LINES, MAX_CHARS, curry, currx, lastoffset;
	char c;
	char buffer[255];
	int offsets[255];

	if (largefont)
		lcd_getfontsize(1, &font_w, &font_h);
	else
		lcd_getfontsize(0, &font_w, &font_h);

    lcd_clear_display();

	MAX_LINES = LCD_HEIGHT / (font_h-2);
	MAX_CHARS = (LCD_WIDTH / font_w)-1;

	offsets[0] = 0;
	curroffset = 1;
	
	/* find line breaks */
	lastoffset = 0;
	for (n=0; n<strlen(caption); n++) {
		c = caption[n];	
		if ((c==' ') || (c==':') || (c=='\\') || (c=='/') || (c=='.') || (c==',')) {
			if (curroffset<MAX_LINES) {
				offsets[curroffset] = n;
			}
		}
		if ((n-lastoffset)>MAX_CHARS) {
			lastoffset = offsets[curroffset];
			curroffset++;
		}
	}
	offsets[curroffset] = strlen(caption);

	curry = (LCD_HEIGHT/2)-((font_h*curroffset)/2);
	
	/* pull out each line */
	for (n=0; n<curroffset; n++) {
		strcpy(buffer, caption + offsets[n]);	
		buffer[(offsets[n+1]-offsets[n])] = 0;
		currx = (LCD_WIDTH/2) - ((font_w*strlen(buffer))/2);
		lcd_putsxy(currx, curry, buffer, largefont ? 1 : 0);
		curry = curry + font_h;
	}
}

char response(char* caption, char* option1text, char* option2text, char* option3text, bool largefont)
{
	int smallfont_w, smallfont_h;
	lcd_getfontsize(0, &smallfont_w, &smallfont_h);

	center_text(caption, largefont);

	if (option1text!=NULL)
		kbd_draw_statusbar_button(0, option1text, LCD_HEIGHT - smallfont_h, smallfont_w);
	if (option2text!=NULL)
		kbd_draw_statusbar_button(1, option2text, LCD_HEIGHT - smallfont_h, smallfont_w);
	if (option3text!=NULL)
		kbd_draw_statusbar_button(2, option3text, LCD_HEIGHT - smallfont_h, smallfont_w);
	
    lcd_update();

	while (1)	{
		sleep(HZ/16);
	    
		switch ( button_get(true) ) {

			case BUTTON_F1:
				if (option1text!=NULL) return option1text[0];
				break;

			case BUTTON_F2:
				if (option2text!=NULL) return option2text[0];
				break;

			case BUTTON_F3:
				if (option3text!=NULL) return option3text[0];
				break;

			case BUTTON_OFF:
				return 0;
				break;

			case BUTTON_PLAY:
				if (option1text!=NULL) return option1text[0];
				break;

		}
	}
}

char* input_string(char* defaulttext)
{
    bool done = false, inCaps = true;

	int font_w = 0, font_h = 0, font_w1 = 0, font_h1 = 0, n;
	int x = 0, y = 0, margin = 0, timer = 0, offsetx = 3, offsety = 3;
	int MAIN_TEXT_X, MAIN_TEXT_Y, MAIN_TEXT_Y2, MAX_TEXT_CHARS;
	int STATUS_BAR_Y1, STATUS_BAR_Y2;
	char* line[KEYBOARD_LINES]; char *text = (char*)malloc(KEYBOARD_BUFFER_SIZE);

	char c = 0;
	
	/* keys are caps to begin with */
	kbd_setupkeys(line, inCaps);

	lcd_getfontsize(0, &font_w, &font_h);
	lcd_getfontsize(1, &font_w1, &font_h1);

	margin = font_h / 2;

	MAIN_TEXT_Y = ((KEYBOARD_LINES+1) * font_h) + offsety;
	MAIN_TEXT_Y2 = MAIN_TEXT_Y + font_h1 + offsety;
	MAIN_TEXT_X = offsetx * 2;
	STATUS_BAR_Y1 = LCD_HEIGHT - font_h;
	STATUS_BAR_Y2 = LCD_HEIGHT;

	MAX_TEXT_CHARS = (LCD_WIDTH/font_w1)-1;
	if (defaulttext!=NULL)
		strcpy(text,  defaulttext);
	else
		strcpy(text,  "");
			   
	while(!done)
    {
        lcd_clear_display();
        
		/* first display all the rows on the keyboard */
		for (n=0; n<KEYBOARD_LINES; n++)
			lcd_putsxy(offsetx, offsety+(n*font_h), line[n], 0);

		/* separator */
		lcd_drawline(0, MAIN_TEXT_Y - offsety, LCD_WIDTH, MAIN_TEXT_Y - offsety);
		lcd_drawline(0, MAIN_TEXT_Y2, LCD_WIDTH, MAIN_TEXT_Y2);
		
		/* write out the text */
		if (strlen(text)<MAX_TEXT_CHARS)
		{	  
			/* if we have enough room */
			lcd_putsxy(MAIN_TEXT_X, MAIN_TEXT_Y, text, 1);
			/* cursor */
			lcd_drawline(MAIN_TEXT_X + (strlen(text) * font_w1), MAIN_TEXT_Y, MAIN_TEXT_X + (strlen(text) * font_w1), MAIN_TEXT_Y + font_h1);
		}
		else
		{
			/* if we don't have enough room, write out the last bit only */
			lcd_putsxy(0, MAIN_TEXT_Y, "<", 0);
			lcd_invertrect(0, MAIN_TEXT_Y, font_w, font_h1);
			
			lcd_putsxy(font_w + 1, MAIN_TEXT_Y, (text + strlen(text)) - (MAX_TEXT_CHARS-1), 1);
			/* cursor */
			lcd_drawline(MAIN_TEXT_X + ((MAX_TEXT_CHARS-1) * font_w1), MAIN_TEXT_Y, MAIN_TEXT_X + ((MAX_TEXT_CHARS-1) * font_w1), MAIN_TEXT_Y + font_h1);
		}
		
		/* draw the status bar */
		kbd_draw_statusbar_button(0, "Shift", STATUS_BAR_Y1, font_w);
		kbd_draw_statusbar_button(1, "OK", STATUS_BAR_Y1, font_w);
		kbd_draw_statusbar_button(2, "Clr", STATUS_BAR_Y1, font_w);
		
		/* highlight the key that has focus */		
		lcd_invertrect(offsetx + (font_w * x),offsety + (font_h * y),font_w,font_h);

		lcd_update();

        sleep(HZ/16);
        
        switch ( button_get(true) ) {

			case BUTTON_OFF:
				/* return null if we cancel the input */
				free(text);
				return NULL;
				break;

			case BUTTON_F1:
				/* toggle upper case / lower case */
				inCaps = inCaps ? false : true;

				/* recreate the keys */
				kbd_setupkeys(line, inCaps);
				break;

            case BUTTON_RIGHT:
                /* move to the right */
				if (x<strlen(line[y])-1) 
					x = x + 1; 
				else 
					x = 0;
                break;

            case BUTTON_LEFT:
                /* move to the left */
				if (x==0) x = strlen(line[y]);
                if (x>0) x = x - 1;
                break;

            case BUTTON_DOWN:
				/* move to the down */
                if (y<(KEYBOARD_LINES-1)) y = y + 1;
                break;

            case BUTTON_UP:
                /* move to the up */
                if (y>0) y = y - 1;
                break;

			case BUTTON_F3:
				/* F3 clears */
				strcpy(text, "");
				break;

			case BUTTON_F2:
				/* F2 accepts what was entered and continues */
				
				/* first resize it */
				text = (char*)realloc(text, strlen(text)+1);
				return text;
				break;

			case BUTTON_PLAY:
				/* PLAY executes the selected key */

				/* get the key with focus */
				c = line[y][x];
				if (c=='<')
				{	  
					/* it's the backspace */
					/* remove the end character */
					if (strlen(text)>0)
						text[strlen(text)-1] = 0;
				}
				else
				{
					/* try to append the key to the text */
					if (strlen(text)<KEYBOARD_BUFFER_SIZE)
					{
						n = strlen(text);
						text[n] = c;
						text[n+1] = 0;
					}
				}
				break;
		}
    }
	return text; 
}


void display_kbd(void)
{
	char *sz = input_string("");
	if (sz) 
	{
		response(sz, "Yes", "No", NULL, false);
		free(sz);
	}
}

